/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "DebugDrawerHelper.h"

#include <VirtualRobot/math/Helpers.h>

#include <RobotAPI/libraries/core/Pose.h>

using namespace math;
using namespace armarx;

DebugDrawerHelper::DebugDrawerHelper(const DebugDrawerInterfacePrx& debugDrawerPrx, const std::string& layerName, const VirtualRobot::RobotPtr& robot)
    : debugDrawerPrx(debugDrawerPrx), robot(robot), rn(robot->getRootNode())
{
}

void DebugDrawerHelper::drawPose(const std::string& name, const Eigen::Matrix4f& pose)
{
    debugDrawerPrx->setPoseVisu(layerName, name, makeGlobal(pose));
}

void DebugDrawerHelper::drawPose(const std::string& name, const Eigen::Matrix4f& pose, float scale)
{
    debugDrawerPrx->setScaledPoseVisu(layerName, name, makeGlobal(pose), scale);
}

void DebugDrawerHelper::drawBox(const std::string& name, const Eigen::Vector3f& position, float size, const DrawColor& color)
{
    drawBox(name, Helpers::CreatePose(position, Eigen::Matrix3f::Identity()), Eigen::Vector3f(size, size, size), color);
}

void DebugDrawerHelper::drawBox(const std::string& name, const Eigen::Matrix4f& pose, float size, const DrawColor& color)
{
    drawBox(name, pose, Eigen::Vector3f(size, size, size), color);
}

void DebugDrawerHelper::drawBox(const std::string& name, const Eigen::Matrix4f& pose, const Eigen::Vector3f& size, const DrawColor& color)
{
    debugDrawerPrx->setBoxVisu(layerName, name, makeGlobal(pose), new Vector3(size), color);
}

void DebugDrawerHelper::drawLine(const std::string& name, const Eigen::Vector3f& p1, const Eigen::Vector3f& p2, float width, const DrawColor& color)
{
    debugDrawerPrx->setLineVisu(layerName, name, makeGlobal(p1), makeGlobal(p2), width, color);
}

void DebugDrawerHelper::drawLine(const std::string& name, const Eigen::Vector3f& p1, const Eigen::Vector3f& p2)
{
    drawLine(name, p1, p2, defaults.lineWidth, defaults.lineColor);
}

void DebugDrawerHelper::drawText(const std::string& name, const Eigen::Vector3f& p1, const std::string& text, const DrawColor& color, int size)
{
    debugDrawerPrx->setTextVisu(layerName, name, text, makeGlobal(p1), color, size);
}

PosePtr DebugDrawerHelper::makeGlobal(const Eigen::Matrix4f& pose)
{
    return new Pose(rn->getGlobalPose(pose));
}

Vector3Ptr DebugDrawerHelper::makeGlobal(const Eigen::Vector3f& position)
{
    return new Vector3(rn->getGlobalPosition(position));
}
