/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <boost/shared_ptr.hpp>

#include <RobotAPI/interface/visualization/DebugDrawerInterface.h>

#include <VirtualRobot/Robot.h>

#include <RobotAPI/libraries/core/Pose.h>

namespace armarx
{
    class DebugDrawerHelper;
    typedef boost::shared_ptr<DebugDrawerHelper> DebugDrawerHelperPtr;

    class DebugDrawerHelper
    {
    public:
        struct Defaults
        {
            float lineWidth = 2;
            DrawColor lineColor = DrawColor {0, 1, 0, 1};
        };

        DebugDrawerHelper(const DebugDrawerInterfacePrx& debugDrawerPrx, const std::string& layerName, const VirtualRobot::RobotPtr& robot);

        void drawPose(const std::string& name, const Eigen::Matrix4f& pose);
        void drawPose(const std::string& name, const Eigen::Matrix4f& pose, float scale);

        void drawBox(const std::string& name, const Eigen::Vector3f& position, float size, const DrawColor& color);
        void drawBox(const std::string& name, const Eigen::Matrix4f& pose, float size, const DrawColor& color);
        void drawBox(const std::string& name, const Eigen::Matrix4f& pose, const Eigen::Vector3f& size, const DrawColor& color);

        void drawLine(const std::string& name, const Eigen::Vector3f& p1, const Eigen::Vector3f& p2, float width, const DrawColor& color);
        void drawLine(const std::string& name, const Eigen::Vector3f& p1, const Eigen::Vector3f& p2);

        void drawText(const std::string& name, const Eigen::Vector3f& p1, const std::string& text, const DrawColor& color, int size);

        PosePtr makeGlobal(const Eigen::Matrix4f& pose);
        Vector3Ptr makeGlobal(const Eigen::Vector3f& position);


        Defaults defaults;

    private:
        DebugDrawerInterfacePrx debugDrawerPrx;
        std::string layerName;
        VirtualRobot::RobotPtr robot;
        VirtualRobot::RobotNodePtr rn;
    };
}
