/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2011-2017, High Performance Humanoid Technologies (H2T), Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    ArmarX
 * @author     Mirko Waechter( mirko.waechter at kit dot edu)
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#pragma once


#include "NJointController.h"
#include <VirtualRobot/Robot.h>
#include "../RobotUnit.h"
#include "../ControlTargets/ControlTarget1DoFActuator.h"
#include "../SensorValues/SensorValue1DoFActuator.h"
#include <VirtualRobot/IK/DifferentialIK.h>
#include <RobotAPI/interface/units/RobotUnit/NJointCartesianActiveImpedanceController.h>
#include <RobotAPI/libraries/core/CartesianPositionController.h>


namespace armarx
{

    TYPEDEF_PTRS_HANDLE(NJointCartesianActiveImpedanceController);
    //TYPEDEF_PTRS_HANDLE(NJointCartesianActiveImpedanceControllerConfig);

    /*class NJointCartesianActiveImpedanceControllerConfig : virtual public NJointControllerConfig
    {
    public:
        NJointCartesianActiveImpedanceControllerConfig(std::string const& nodeSetName, const std::string& tcpName, VirtualRobot::IKSolver::CartesianSelection mode):
            nodeSetName(nodeSetName),
            tcpName(tcpName),
            mode(mode)
        {}

        const std::string nodeSetName;
        const std::string tcpName;
        const VirtualRobot::IKSolver::CartesianSelection mode;
    };*/

    TYPEDEF_PTRS_HANDLE(NJointCartesianActiveImpedanceControllerControlData);
    class NJointCartesianActiveImpedanceControllerControlData
    {
    public:
        float xPos = 0;
        float yPos = 0;
        float zPos = 0;
        float qw = 0, qx = 0, qy = 0, qz = 0;
        float KpPos = 0;
        float KpOri = 0;
    };

    class SimplePID
    {
    public:
        float Kp = 0, Kd = 0;
        float lastError = 0;
        float update(float dt, float error);
    };


    class NJointCartesianActiveImpedanceController :
        public NJointControllerWithTripleBuffer<NJointCartesianActiveImpedanceControllerControlData>,
        public NJointCartesianActiveImpedanceControllerInterface
    {
    public:
        using ConfigPtrT = NJointCartesianActiveImpedanceControllerConfigPtr;
        NJointCartesianActiveImpedanceController(NJointControllerDescriptionProviderInterfacePtr prov, const NJointControllerConfigPtr& config, const VirtualRobot::RobotPtr&);

        // NJointControllerInterface interface
        std::string getClassName(const Ice::Current&) const override;
        WidgetDescription::StringWidgetDictionary getFunctionDescriptions(const Ice::Current&) const override;
        void callDescribedFunction(const std::string& name, const StringVariantBaseMap& valueMap, const Ice::Current&) override;

        // NJointController interface
        void rtRun(const IceUtil::Time& sensorValuesTimestamp, const IceUtil::Time& timeSinceLastIteration) override;
        static WidgetDescription::WidgetPtr GenerateConfigDescription(
            const boost::shared_ptr<VirtualRobot::Robot>&,
            const std::map<std::string, ConstControlDevicePtr>&,
            const std::map<std::string, ConstSensorDevicePtr>&);

        static NJointCartesianActiveImpedanceControllerConfigPtr GenerateConfigFromVariants(const StringVariantBaseMap& values);
        NJointCartesianActiveImpedanceController(
            NJointControllerDescriptionProviderInterfacePtr prov,
            NJointCartesianActiveImpedanceControllerConfigPtr config,
            const boost::shared_ptr<VirtualRobot::Robot>& r);

        std::string getNodeSetName() const;
        static ::armarx::WidgetDescription::WidgetSeq createSliders();
        WidgetDescription::HBoxLayoutPtr createJointSlidersLayout(float min, float max, float defaultValue) const;

    protected:
        void rtPreActivateController() override;
        void rtPostDeactivateController() override;
    private:
        std::vector<ControlTarget1DoFActuatorVelocity*> targets;
        //std::vector<const SensorValue1DoFActuatorPosition*> sensors;
        std::vector<const SensorValue1DoFActuatorTorque*> torqueSensors;
        std::vector<const SensorValue1DoFGravityTorque*> gravityTorqueSensors;
        std::vector<SimplePID> torquePIDs;

        CartesianPositionControllerPtr tcpController;

        std::string nodeSetName;
        VirtualRobot::DifferentialIKPtr ik;
        VirtualRobot::RobotNodePtr tcp;

        Eigen::Matrix4f targetPose;


        // NJointCartesianActiveImpedanceControllerInterface interface
    public:
        void setControllerTarget(float x, float y, float z, float qw, float qx, float qy, float qz, const Ice::Current&);
    };

} // namespace armarx

