/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::RobotUnit
 * @author     Raphael Grimm ( raphael dot grimm at kit dot edu )
 * @date       2018
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include "RobotUnitModuleBase.h"

namespace armarx
{
    namespace RobotUnitModule
    {

        class ManagementPropertyDefinitions: public ModuleBasePropertyDefinitions
        {
        public:
            ManagementPropertyDefinitions(std::string prefix): ModuleBasePropertyDefinitions(prefix)
            {
                defineOptionalProperty<std::uint64_t>(
                    "AdditionalObjectSchedulerCount", 10,
                    "Number of ObjectSchedulers to be added");
            }
        };

        /**
         * @ingroup Library-RobotUnit-Modules
         * @brief This \ref ModuleBase "Module" handles some general management tasks.
         * It implements the \ref RobotUnitManagementInterface.
         *
         * @see ModuleBase
         */
        class Management : virtual public ModuleBase, virtual public RobotUnitManagementInterface
        {
            friend class ModuleBase;
        public:
            /**
             * @brief Returns the singleton instance of this class
             * @return The singleton instance of this class
             */
            static Management& Instance()
            {
                return ModuleBase::Instance<Management>();
            }
            // //////////////////////////////////////////////////////////////////////////////////////// //
            // ///////////////////////////////// RobotUnitModule hooks //////////////////////////////// //
            // //////////////////////////////////////////////////////////////////////////////////////// //
        private:
            /// @see ModuleBase::_preOnInitRobotUnit
            void _preOnInitRobotUnit();
            // //////////////////////////////////////////////////////////////////////////////////////// //
            // ///////////////////////////////////// ice interface //////////////////////////////////// //
            // //////////////////////////////////////////////////////////////////////////////////////// //
        public:
            /**
             * @brief Returns whether the RobotUnit is running.
             * @return Whether the RobotUnit is running.
             */
            bool isRunning(const Ice::Current& = GlobalIceCurrent) const override
            {
                throwIfInControlThread(BOOST_CURRENT_FUNCTION);
                return getRobotUnitState() == RobotUnitState::Running;
            }
            // //////////////////////////////////////////////////////////////////////////////////////// //
            // ///////////////////////////////////////// Data ///////////////////////////////////////// //
            // //////////////////////////////////////////////////////////////////////////////////////// //
        private:
            /// @brief The number of additional object schedulers
            std::uint64_t additionalObjectSchedulerCount {0};
        };
    }
}
