/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <RobotAPI/interface/core/RobotState.h>

#include <ArmarXCore/statechart/xmlstates/profiles/StatechartProfiles.h>

#include <boost/optional.hpp>

#include <VirtualRobot/Robot.h>

namespace armarx
{
    typedef std::shared_ptr<class RobotNameHelper> RobotNameHelperPtr;

    class RobotNameHelper : public std::enable_shared_from_this<RobotNameHelper>
    {
    public:
        class Node
        {
        public:
            Node(const RobotInfoNodePtr& robotInfo);
            std::string value();

            Node select(const std::string& name, const std::vector<std::string>& profiles);
            bool isValid();
            void checkValid();

        private:
            RobotInfoNodePtr robotInfo;
        };

        static const std::string LocationLeft;
        static const std::string LocationRight;

        struct RobotArm;
        struct Arm
        {
            friend class RobotNameHelper;
        public:

            std::string getKinematicChain() const;
            std::string getTorsoKinematicChain() const;
            std::string getTCP() const;
            std::string getForceTorqueSensor() const;
            std::string getEndEffector() const;
            std::string getMemoryHandName() const;
            std::string getHandControllerName() const;
            RobotArm addRobot(const VirtualRobot::RobotPtr& robot) const;

        private:
            Arm(const std::shared_ptr<RobotNameHelper>& rnh, const std::string& side);
            std::string select(const std::string& path) const;

            std::shared_ptr<RobotNameHelper> rnh;
            std::string side;
        };

        struct RobotArm
        {
            friend class RobotNameHelper;
            friend class Arm;
        public:
            VirtualRobot::RobotNodeSetPtr getKinematicChain() const;
            VirtualRobot::RobotNodeSetPtr getTorsoKinematicChain() const;
            VirtualRobot::RobotNodePtr getTCP() const;

        private:
            RobotArm(const Arm& arm, const VirtualRobot::RobotPtr& robot);

            const Arm arm;
            VirtualRobot::RobotPtr robot;
        };

        std::string select(const std::string& path) const;

        static RobotNameHelperPtr Create(const RobotInfoNodePtr& robotInfo, const StatechartProfilePtr& profile);

        Arm getArm(const std::string& side);
        RobotArm getRobotArm(const std::string& side, const VirtualRobot::RobotPtr& robot);

    private:
        RobotNameHelper(const RobotInfoNodePtr& robotInfo, const StatechartProfilePtr& profile);
        RobotNameHelper& self()
        {
            return *this;
        }

        Node root;
        std::vector<std::string> profiles;


    };
}
