/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Sonja Marahrens (sonja dot marahrens)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <boost/shared_ptr.hpp>
#include <Eigen/Dense>

namespace armarx
{
    class JointVelocityRamp;
    typedef boost::shared_ptr<JointVelocityRamp> JointVelocityRampPtr;

    class JointVelocityRamp
    {
    public:
        JointVelocityRamp();

        void setState(const Eigen::VectorXf& state);
        Eigen::VectorXf update(const Eigen::VectorXf& target, float dt);

        void setMaxAccelaration(float maxAcceleration);
        float getMaxAccelaration();

    private:

        Eigen::VectorXf state;
        float maxAcceleration;

    };
}
