/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @package    ArmarX::RobotAPI::Test
* @author     Nils Adermann (naderman at naderman dot de)
* @date       2010
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#define BOOST_TEST_MODULE RobotAPI::CartesianVelocityController::Test
#define ARMARX_BOOST_TEST
#include <RobotAPI/Test.h>
#include <ArmarXCore/core/test/IceTestHelper.h>
#include <ArmarXCore/core/system/ArmarXDataPath.h>
#include <VirtualRobot/XML/RobotIO.h>
#include <ArmarXCore/core/system/cmake/CMakePackageFinder.h>
#include "../CartesianVelocityController.h"

using namespace armarx;

void check_close(float a, float b, float tolerance)
{
    if (fabs(a - b) > tolerance)
    {
        ARMARX_ERROR << "fabs(a-b) > tolerance. a=" << a << " b=" << b << " tolerance=" << tolerance;
        BOOST_FAIL("");
    }
}




BOOST_AUTO_TEST_CASE(test1)
{
    const std::string project = "RobotAPI";
    armarx::CMakePackageFinder finder(project);

    if (!finder.packageFound())
    {
        ARMARX_ERROR_S << "ArmarX Package " << project << " has not been found!";
    }
    else
    {
        armarx::ArmarXDataPath::addDataPaths(finder.getDataDir());
    }
    std::string fn = "RobotAPI/robots/Armar3/ArmarIII.xml";
    ArmarXDataPath::getAbsolutePath(fn, fn);
    std::string robotFilename = fn;
    VirtualRobot::RobotPtr robot = VirtualRobot::RobotIO::loadRobot(robotFilename, VirtualRobot::RobotIO::eStructure);
    VirtualRobot::RobotNodeSetPtr rns = robot->getRobotNodeSet("HipYawRightArm");
    CartesianVelocityControllerPtr h(new CartesianVelocityController(rns));
    Eigen::VectorXf jointVel(8);
    jointVel << 1, 1, 1, 1, 1, 1, 1, 1;

    Eigen::VectorXf targetTcpVel(6);
    targetTcpVel << 0, 0, 0, 0, 0, 0;

    ARMARX_IMPORTANT << rns->getJointValues();
    Eigen::VectorXf vel = h->calculate(targetTcpVel, jointVel, VirtualRobot::IKSolver::CartesianSelection::All);
    ARMARX_IMPORTANT << vel.transpose();
    Eigen::VectorXf tcpVel = (h->jacobi * vel).transpose();
    ARMARX_IMPORTANT << tcpVel;
    BOOST_CHECK_LE((targetTcpVel - tcpVel).norm(), 0.01f);

}
